# Overview

This is an artefact for the FORTE 2024 submission "Explicit Contexts for Substructural Systems". The artefact contains encodings in [Beluga](https://github.com/Beluga-lang/Beluga) of the propositional fragment of [CP](https://dl.acm.org/doi/10.1145/2398856.2364568): one using higher-order abstract syntax (CPᵛ), the other using de Bruijn indices (CPⁿ). We implement proofs of their equivalence to each other and to an encoding of [Structural CP](https://dl.acm.org/doi/10.1145/3622810). We also mechanise proofs of type preservation for each encoding.


## Structure

To ease navigation, the artefact is organised into folders by theme: `common` contains definitions and lemmata shared by both encodings. `cpn` and `cpv` contain the encodings we call CPⁿ and CPᵛ, respectively, lemmata related to each, and proofs of type preservation. The encoding of SCP due to Sano et al. may be found in `scp`, and the equivalence proofs in `equiv`. Below is a more detailed breakdown:

	cp.cfg                    - Collects Beluga source files

	common/common.bel         - Encoding of multiplicites, session types, natural numbers, variables
	common/lemmata.bel        - Proofs of common lemmata

	cpn/proc.bel              - Encoding of CPⁿ processes and operations on processes
	cpn/ctx.bel               - Encoding of context infrastructure for CPⁿ
	cpn/statics.bel           - Encoding of typing rules for CPⁿ
	cpn/dyn.bel               - Encoding of dynamics of CPⁿ
	cpn/lemmata/nats.bel      - Proofs of lemmata about natural numbers
	cpn/lemmata/size.bel      - Proofs of lemmata about the size of contexts for CPⁿ
	cpn/lemmata/ins.bel       - Proofs of lemmata about name insertion for CPⁿ
	cpn/lemmata/merge.bel     - Proofs of lemmata about context merging for CPⁿ
	cpn/lemmata/cp.bel        - Proofs of lemmata about CPⁿ processes and typing judgments
	cpn/thms.bel              - Proof of type preservation for CPⁿ

	cpv/proc.bel              - Encoding of CPᵛ processes and operations on processes
	cpv/ctx.bel               - Encoding of context infrastructure for CPᵛ
	cpv/statics.bel           - Encoding of typing rules for CPᵛ
	cpv/dyn.bel               - Encoding of dynamics of CPᵛ
	cpv/lemmata/vars.bel      - Proofs of lemmata about variables for CPᵛ
	cpv/lemmata/upd.bel       - Proofs of lemmata about context updating for CPᵛ
	cpv/lemmata/merge.bel     - Proofs of lemmata about context merging for CPᵛ
	cpv/lemmata/cp.bel        - Proofs of lemmata about CPᵛ processes and typing judgments
	cpv/thms.bel              - Proof of type preservation for CPᵛ

	scp/linear.bel            - Encoding of SCP linearity predicate
	scp/statics.bel           - Encoding of typing rules for SCP
	scp/lemmata.bel           - Proofs of lemmata about SCP

	equiv/transl.bel          - Encoding of translation between CPⁿ, CPᵛ, and SCP
	equiv/lemmata/cpn_cpv.bel - Proofs of lemmata for translation between CPⁿ and CPᵛ
	equiv/lemmata/cpv_scp.bel - Proofs of lemmata for translation from CPᵛ to SCP
	equiv/lemmata/scp_cpv.bel - Proofs of lemmata for translation from SCP to CPᵛ
	equiv/thms.bel            - Proof of equivalence of CPⁿ, CPᵛ, and SCP, and properties of the translation


## Paper-to-artifact correspondence guide

| Definition / theorem                    | Paper         | File                      | Definition name                                        |
|-----------------------------------------|---------------|---------------------------|--------------------------------------------------------|
| Typing contexts                         | §3            | cpn/ctx.bel               | lctxⁿ                                                  |
|                                         |               | cpv/ctx.bel               | lctxᵛ                                                  |
| Well-formedness predicate               | §3.1          | cpv/ctx.bel               | Wf                                                     |
| Multiplicities, multiplicity relation   | §3.2          | common/common.bel         | mult, •                                                |
| Context merge                           | §3.2          | cpn/ctx.bel               | mergeⁿ                                                 |
|                                         |               | cpv/ctx.bel               | mergeᵛ                                                 |
| Algebraic properties of merge           | §3.2, Prop. 1 | cpn/lemmata/merge.bel     | merge_idⁿ, merge_assocⁿ, merge_commⁿ, merge_canclⁿ     |
|                                         |               | cpv/lemmata/merge.bel     | merge_idᵛ, merge_assocᵛ, merge_commᵛ, merge_canclᵛ     |
| Insertion                               | §3.3          | cpn/ctx.bel               | insⁿ                                                   |
| Updating, removal                       | §3.3          | cpv/ctx.bel               | updᵛ, rmvᵛ                                             |
| Algebraic properties of insertion       | §3.3, Prop. 2 | cpn/lemmata/ins.bel       | ins_runq, ins_lunq, ins_comm, rmv_comm                 |
|                                         |               | cpn/lemmata/merge.bel     | merge_strⁿ, merge_weakⁿ                                |
| Algebraic properties of updating        | §3.3, Prop. 3 | cpv/lemmata/upd.bel       | upd_reflᵛ, upd_symmᵛ, upd_transᵛ, upd_confᵛ, upd_runqᵛ |
|                                         |               | cpv/lemmata/merge.bel     | merge_updᵛ                                             |
| Session types, duality                  | §4.1          | common/common.bel         | tp, dual                                               |
| Processes                               | §4.1          | cpn/proc.bel              | procⁿ                                                  |
|                                         |               | cpv/proc.bel              | procᵛ                                                  |
| Typing judgment                         | §4.1          | cpn/statics.bel           | oftⁿ                                                   |
|                                         |               | cpv/statics.bel           | oftᵛ                                                   |
| Properties of typing judgments          | §4.2, Lemma 1 | cpn/lemmata/cp.bel        | oft_exchⁿ, oft_strⁿ, oft_weakⁿ, oft_renameⁿ            |
|                                         |               | cpv/lemmata/cp.bel        | oft_exchᵛ, oft_strᵛ, oft_weakᵛ, oft_renameᵛ            |
| Reductions                              | §4.5          | cpn/dyn.bel               | ⇛ⁿ                                                     |
|                                         |               | cpv/dyn.bel               | ⇛ᵛ                                                     |
| Translation predicates                  | §4.3–4        | equiv/transl.bel          | EncCtxⁿᵛ, enc, EncCtxᵛˢ, DecCtxᵛˢ                      |
| Translating manipulation judgments      | §4.3, Lemma 2 | equiv/lemmata/cpn_cpv.bel | transl_upd_num                                         |
| Invertibility of translation predicates | §4.4, Prop. 4 | equiv/thms.bel            | enc_to_dec, dec_to_enc                                 |
| Extracting linearity predicate          | §4.4, Lemma 3 | equiv/lemmata/cpv_scp.bel | oft_linearᵛ                                            |
| Equivalence of CPⁿ and CPᵛ              | §4.3, Thm. 1  | equiv/thms.bel            | cpn_to_cpv, cpv_to_cpn                                 |
| Equivalence of CPᵛ and SCP              | §4.4, Thm. 2  | equiv/thms.bel            | cpv_to_scp, scp_to_cpv                                 |
| Type preservation                       | §4.5, Thm. 3  | cpn/thms.bel              | tp_pres_cpⁿ                                            |
|                                         |               | cpv/thms.bel              | tp_pres_cpᵛ                                            |


# Installation and execution

This mechanisation is compatible with Beluga version 1.1.1.


## Installation

Beluga may be installed using the OCaml package manager ([`opam`](https://opam.ocaml.org/doc/Install.html)):

	>> opam install beluga.1.1.1

It may also be built and installed from source following the instructions at https://github.com/Beluga-lang/Beluga.

Alternatively, we have provided a virtual machine based on OpenBSD 7.4 that has Beluga 1.1.1, its sources, and our mechanization pre-installed.
This QCOW2 disk image can be run using QEMU using the command:

    >> qemu-system-amd64 -hda artifact.qcow2 -m 1G

For a better user experience, we suggest the following options, which give the virtual machine 4 gigabytes of memory, four CPUs, and allow the machine to be accessed over SSH on localhost port 60022 using username `artifact` and password `artifact`:

    >> qemu-system-amd64 -hda artifact.qcow2 -m 4G -smp 4 \
                         -nic user,hostfwd=tcp::60022-:22
    >> ssh artifact@127.0.0.1 -p 60022

## Execution

Once installed, Beluga can be run on the file `cp.cfg`. The expected total runtime is approximately 1m40s, and the expected output as follows.

	>> beluga cp.cfg
	## Type Reconstruction begin: ./common/common.bel ##
	## Type Reconstruction done:  ./common/common.bel ##
	## Type Reconstruction begin: ./common/lemmata.bel ##
	## Type Reconstruction done:  ./common/lemmata.bel ##
	## Type Reconstruction begin: ./cpn/proc.bel ##
	## Type Reconstruction done:  ./cpn/proc.bel ##
	## Type Reconstruction begin: ./cpn/ctx.bel ##
	## Type Reconstruction done:  ./cpn/ctx.bel ##
	## Type Reconstruction begin: ./cpn/statics.bel ##
	## Type Reconstruction done:  ./cpn/statics.bel ##
	## Type Reconstruction begin: ./cpn/dyn.bel ##
	## Type Reconstruction done:  ./cpn/dyn.bel ##
	## Type Reconstruction begin: ./cpn/lemmata/nats.bel ##
	## Type Reconstruction done:  ./cpn/lemmata/nats.bel ##
	## Type Reconstruction begin: ./cpn/lemmata/size.bel ##
	## Type Reconstruction done:  ./cpn/lemmata/size.bel ##
	## Type Reconstruction begin: ./cpn/lemmata/ins.bel ##
	## Type Reconstruction done:  ./cpn/lemmata/ins.bel ##
	## Type Reconstruction begin: ./cpn/lemmata/merge.bel ##
	## Type Reconstruction done:  ./cpn/lemmata/merge.bel ##
	## Type Reconstruction begin: ./cpn/lemmata/cp.bel ##
	## Type Reconstruction done:  ./cpn/lemmata/cp.bel ##
	## Type Reconstruction begin: ./cpv/proc.bel ##
	## Type Reconstruction done:  ./cpv/proc.bel ##
	## Type Reconstruction begin: ./cpv/ctx.bel ##
	## Type Reconstruction done:  ./cpv/ctx.bel ##
	## Type Reconstruction begin: ./cpv/statics.bel ##
	## Type Reconstruction done:  ./cpv/statics.bel ##
	## Type Reconstruction begin: ./cpv/dyn.bel ##
	## Type Reconstruction done:  ./cpv/dyn.bel ##
	## Type Reconstruction begin: ./cpv/lemmata/vars.bel ##
	## Type Reconstruction done:  ./cpv/lemmata/vars.bel ##
	## Type Reconstruction begin: ./cpv/lemmata/upd.bel ##
	## Type Reconstruction done:  ./cpv/lemmata/upd.bel ##
	## Type Reconstruction begin: ./cpv/lemmata/merge.bel ##
	## Type Reconstruction done:  ./cpv/lemmata/merge.bel ##
	## Type Reconstruction begin: ./cpv/lemmata/cp.bel ##
	## Type Reconstruction done:  ./cpv/lemmata/cp.bel ##
	## Type Reconstruction begin: ./scp/linear.bel ##
	## Type Reconstruction done:  ./scp/linear.bel ##
	## Type Reconstruction begin: ./scp/statics.bel ##
	## Type Reconstruction done:  ./scp/statics.bel ##
	## Type Reconstruction begin: ./scp/lemmata.bel ##
	## Type Reconstruction done:  ./scp/lemmata.bel ##
	## Type Reconstruction begin: ./equiv/transl.bel ##
	## Type Reconstruction done:  ./equiv/transl.bel ##
	## Type Reconstruction begin: ./equiv/lemmata/cpn_cpv.bel ##
	## Type Reconstruction done:  ./equiv/lemmata/cpn_cpv.bel ##
	## Type Reconstruction begin: ./equiv/lemmata/cpv_scp.bel ##
	## Type Reconstruction done:  ./equiv/lemmata/cpv_scp.bel ##
	## Type Reconstruction begin: ./equiv/lemmata/scp_cpv.bel ##
	## Type Reconstruction done:  ./equiv/lemmata/scp_cpv.bel ##
	## Type Reconstruction begin: ./cpn/thms.bel ##
	## Type Reconstruction done:  ./cpn/thms.bel ##
	## Type Reconstruction begin: ./cpv/thms.bel ##
	## Type Reconstruction done:  ./cpv/thms.bel ##
	## Type Reconstruction begin: ./equiv/thms.bel ##
	## Type Reconstruction done:  ./equiv/thms.bel ##

**Remark**: Beluga's totality checker is memory-intensive.
Depending on your system's available resources, it may cause Beluga to abort during OCaml's garbage collection run:

    >> beluga cp.cfg
    Fatal error: allocation failure during minor GC
    Abort trap (core dumped)

If this occurs, you may disable the totality check on the most memory-intensive theorem by applying the patch `disable_totality.diff`:

    >> cat disable_totality.diff
	diff -Nurp old/cpn/lemmata/cp.bel new/cpn/lemmata/cp.bel
	--- old/cpn/lemmata/cp.bel      Fri Mar  1 20:00:31 2024
	+++ new/cpn/lemmata/cp.bel      Thu Mar  7 14:48:04 2024
	@@ -3542,7 +3541,6 @@ rec oft_renameⁿ : [ ⊢ oftⁿ P Δ] → [ ⊢ notin
	 rec oft_exchⁿ : [ ⊢ oftⁿ P Δ]
	 → [ ⊢ insⁿ Δₙ n A α Δ] → [ ⊢ insⁿ Δₙ n B β Δ''] → [ ⊢ insⁿ Δₘ m B β Δ''] → [ ⊢ insⁿ Δₘ m A α Δ']
	 → [ ⊢ swap_in_proc P n m P'] → [ ⊢ oftⁿ P' Δ'] =
	-  / total 1 /
	   fn OFT, INSn1, INSn2, INSm1, INSm2, SW ⇒
	   let [ ⊢ swap-in-proc CH1 CH2 CH3] = SW in
	   let [ ⊢ INSn1']:[ ⊢ insⁿ _ n C _ _] = INSn1 in
    $ patch -p1 < disable_totality.diff
